import torch
from torch.utils.data import Dataset
import os
import numpy as np
import matplotlib.pyplot as plt
import nibabel as nib
import random
import pandas as pd
from REMOVE_ADNI_SUBJECTS import *


class ADNIDataset(torch.utils.data.Dataset):
    """3D MRI dataset loader- ADNI"""

    def __init__(self, image_path='/data/ADNI_carole',
                 mask_path=None,
                 mask_brain_path=None,
                 num_classes=2,
                 class_label=0,
                 get_id=False,
                 transform=None):
        """
        Args:
            image_path (string): path to image folder.
            label_path (string): path to labels file.
            transform (callable, optional): Optional transforms to be applied on a sample.
        """
        self.transform = transform
        self.num_classes = num_classes
        self.class_label = class_label
        self.img_dir = image_path
        self.image_paths = sorted(os.listdir(self.img_dir))
        self.mask_dir = mask_path
        self.mask_brain_path = mask_brain_path
        self.get_id = get_id

    def __len__(self):
        return len(self.image_paths)

    def __getitem__(self, idx, plot=False):
        img_name = sorted(self.image_paths)[idx]

        image = nib.load(os.path.join(self.img_dir, img_name))
        image = np.float32(image.get_fdata())

        if not self.mask_dir is None:
            mask = nib.load(os.path.join(self.mask_dir, img_name))
            mask = np.float32(mask.get_fdata())
        if not self.mask_brain_path is None:
            mask_brain = nib.load(os.path.join(self.mask_brain_path, img_name))
            mask_brain = np.float32(mask_brain.get_fdata())

        if plot:
            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]
            fig = plt.figure()
            a = fig.add_subplot(2, 3, 1)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('Before aug')
            a = fig.add_subplot(2, 3, 2)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 3)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')

            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

        if self.transform:
            image = self.transform(image)
            if not self.mask_dir is None:
                mask = self.transform(mask)
            if not self.mask_brain_path is None:
                mask_brain = self.transform(mask_brain)

        if plot:
            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]

            a = fig.add_subplot(2, 3, 4)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('After aug')
            a = fig.add_subplot(2, 3, 5)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 6)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')
            plt.show()

        image = torch.from_numpy(image.copy()).float()
        image = image.unsqueeze(0)
        if not self.mask_dir is None:
            mask = torch.from_numpy(mask.copy()).float()
            mask = mask.unsqueeze(0)
        else:
            mask = torch.zeros((1))

        label = np.zeros(self.num_classes)
        label[self.class_label] = 1

        if self.get_id:
            label = [label, img_name.split('.')[0]]
            if not self.mask_brain_path is None:
                mask_brain = torch.from_numpy(mask_brain.copy()).float()
                mask_brain = mask_brain.unsqueeze(0)
                label = label + [mask_brain]

        return image, label, mask

class ADNIConvSubDataset(torch.utils.data.Dataset):
    """3D MRI dataset loader- ADNI"""

    def __init__(self, image_path='/data/ADNI_carole',
                 mask_path=None,
                 mask_brain_path=None,
                 num_classes=2,
                 class_label=0,
                 get_id=False,
                 group='train',
                 labels_path='/data/ADNI_carole/ConversionProject/ADNI_mmse_date_age_labels.pkl',
                 regression=False,
                 reg_content=False,
                 val_train_sub_split=True,
                 transform=None):
        """
        Args:
            image_path (string): path to image folder.
            label_path (string): path to labels file.
            transform (callable, optional): Optional transforms to be applied on a sample.
        """
        self.transform = transform
        self.num_classes = num_classes
        self.group = group
        self.class_label = class_label
        self.img_dir = image_path
        self.image_paths = sorted(os.listdir(self.img_dir))
        temp_paths = sorted(os.listdir(self.img_dir))
        self.mask_dir = mask_path
        self.mask_brain_path = mask_brain_path
        self.get_id = get_id
        self.regression = regression
        self.reg_content = reg_content
        self.labels = pd.read_pickle(labels_path)
        # print(sorted(self.image_paths))
        # print(len(self.image_paths))

        if regression:
            # # print(len(self.image_paths))
            # REMOVE_AD_MCI_NO_MMSE_SUB = REMOVE_AD_NO_MMSE_SUB + REMOVE_MCI_NO_MMSE_SUB
            # for im in temp_paths:
            #     image_name = im.split('.')[0]
            #     if any(image_name in s for s in REMOVE_AD_MCI_NO_MMSE_SUB):
            #         try:
            #             self.image_paths.remove(im)
            #         except:
            #             continue

            # print(len(self.image_paths))
            id_list = sorted(self.image_paths)
            remove_ind = []
            i = 0
            # check which images are present in labels
            for img in id_list:
                subject = img.split('.')[0]
                if not (any(self.labels['id'] == subject)):
                    remove_ind.append(i)
                i = i + 1
            id_list = [i for j, i in enumerate(id_list) if j not in remove_ind]
            self.image_paths = sorted(id_list)
            # print(len(self.image_paths))

            MMSE_scores = np.zeros(0)
            for im in self.image_paths:
                image_name = im.split('.')[0]
                labels = self.labels.loc[self.labels['id'] == image_name]
                label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])
                MMSE_scores = np.append(MMSE_scores, label)

            # print(MMSE_scores.mean(), MMSE_scores.std())

        # print(len(self.image_paths))
        if group == 'train':
            for im in temp_paths:
                image_name = im.split('.')[0]
                if any(image_name in s for s in REMOVE_TEST_IDS):
                    try:
                        self.image_paths.remove(im)
                    except:
                        continue
        elif group == 'val':
            if val_train_sub_split:
                for im in temp_paths:
                    image_name = im.split('.')[0]
                    if any(image_name in s for s in TEST_SUB):
                        try:
                            self.image_paths.remove(im)
                        except:
                            continue
        else:
            if val_train_sub_split:
                for im in temp_paths:
                    image_name = im.split('.')[0]
                    if any(image_name in s for s in VAL_SUB):
                        try:
                            self.image_paths.remove(im)
                        except:
                            continue

        if regression:
            MMSE_scores = np.zeros(0)
            for im in self.image_paths:
                image_name = im.split('.')[0]
                labels = self.labels.loc[self.labels['id'] == image_name]
                label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])
                MMSE_scores = np.append(MMSE_scores, label)

            # print(MMSE_scores.mean(), MMSE_scores.std())

        # print(len(self.image_paths))
        # print(sorted(self.image_paths))

    def __len__(self):
        return len(self.image_paths)

    def __getitem__(self, idx, plot=False):
        img_name = sorted(self.image_paths)[idx]

        # print(img_name)

        image = nib.load(os.path.join(self.img_dir, img_name))
        image = np.float32(image.get_fdata())

        if not self.mask_dir is None:
            mask = nib.load(os.path.join(self.mask_dir, img_name))
            mask = np.float32(mask.get_fdata())
        if not self.mask_brain_path is None:
            mask_brain = nib.load(os.path.join(self.mask_brain_path, img_name))
            mask_brain = np.float32(mask_brain.get_fdata())

        if plot:
            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]
            fig = plt.figure()
            a = fig.add_subplot(2, 3, 1)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('Before aug')
            a = fig.add_subplot(2, 3, 2)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 3)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')

            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

        if self.transform:
            image = self.transform(image)
            if not self.mask_dir is None:
                mask = self.transform(mask)
            if not self.mask_brain_path is None:
                mask_brain = self.transform(mask_brain)

        if plot:
            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]

            a = fig.add_subplot(2, 3, 4)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('After aug')
            a = fig.add_subplot(2, 3, 5)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 6)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')
            plt.show()

        image = torch.from_numpy(image.copy()).float()
        image = image.unsqueeze(0)
        if not self.mask_dir is None:
            mask = torch.from_numpy(mask.copy()).float()
            mask = mask.unsqueeze(0)
        else:
            mask = torch.zeros((1))


        if not self.regression:
            label = np.zeros(self.num_classes)
            label[self.class_label] = 1

            if self.get_id:
                label = [label, img_name.split('.')[0]]
                if not self.mask_brain_path is None:
                    mask_brain = torch.from_numpy(mask_brain.copy()).float()
                    mask_brain = mask_brain.unsqueeze(0)
                    label = label + [mask_brain]

        else:
            f = img_name.split('.')[0]
            labels = self.labels.loc[self.labels['id'] == f]
            if self.class_label == 0:
                if self.group == 'train':
                    labels = labels.loc[labels['group'] == 'MCI']
                else:
                    labels = labels.loc[labels['group'] == 'Conv_MCI']
                label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])
                if self.reg_content:
                    age =np.array([labels['age'].to_numpy().astype(float)[0]])
            else:
                if self.group == 'train':
                    labels = labels.loc[labels['group'] == 'AD']
                else:
                    labels = labels.loc[labels['group'] == 'Conv_AD']
                label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])
                if self.reg_content:
                    age =np.array([labels['age'].to_numpy().astype(float)[0]])

            label = torch.from_numpy(label.copy()).float()
            if self.reg_content:
                label = [label, age]

            if self.get_id:
                if self.reg_content:
                    label = label + [img_name.split('.')[0]]
                else:
                    label = [label, img_name.split('.')[0]]
                if not self.mask_brain_path is None:
                    mask_brain = torch.from_numpy(mask_brain.copy()).float()
                    mask_brain = mask_brain.unsqueeze(0)
                    label = label + [mask_brain]

        return image, label, mask


class ADNIConvSubDataset_2(torch.utils.data.Dataset):
    """3D MRI dataset loader- ADNI"""

    def __init__(self, image_path='/data/ADNI_carole',
                 mask_path=None,
                 mask_brain_path=None,
                 num_classes=2,
                 class_label=0,
                 get_id=False,
                 group='train',
                 transform=None):
        """
        Args:
            image_path (string): path to image folder.
            label_path (string): path to labels file.
            transform (callable, optional): Optional transforms to be applied on a sample.
        """
        self.transform = transform
        self.num_classes = num_classes
        self.class_label = class_label
        self.img_dir = image_path
        self.image_paths = sorted(os.listdir(self.img_dir))
        temp_paths = sorted(os.listdir(self.img_dir))
        self.mask_dir = mask_path
        self.mask_brain_path = mask_brain_path
        self.get_id = get_id

        # print(len(self.image_paths))
        if group == 'train':
            for im in temp_paths:
                image_name = im.split('.')[0]
                if any(image_name in s for s in REMOVE_TEST_IDS):
                    self.image_paths.remove(im)
        else:
            for im in temp_paths:
                image_name = im.split('.')[0]
                if any(image_name in s for s in REMOVE_CONV_SUB):
                    self.image_paths.remove(im)
        # print(len(self.image_paths))
        # print(self.image_paths)

    def __len__(self):
        return len(self.image_paths)

    def __getitem__(self, idx, plot=False):
        img_name = sorted(self.image_paths)[idx]

        print(img_name)

        image = nib.load(os.path.join(self.img_dir, img_name))
        image = np.float32(image.get_fdata())

        if not self.mask_dir is None:
            mask = nib.load(os.path.join(self.mask_dir, img_name))
            mask = np.float32(mask.get_fdata())
        if not self.mask_brain_path is None:
            mask_brain = nib.load(os.path.join(self.mask_brain_path, img_name))
            mask_brain = np.float32(mask_brain.get_fdata())

        if plot:
            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]
            fig = plt.figure()
            a = fig.add_subplot(2, 3, 1)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('Before aug')
            a = fig.add_subplot(2, 3, 2)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 3)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')

            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

        if self.transform:
            image = self.transform(image)
            if not self.mask_dir is None:
                mask = self.transform(mask)
            if not self.mask_brain_path is None:
                mask_brain = self.transform(mask_brain)

        if plot:
            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]

            a = fig.add_subplot(2, 3, 4)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('After aug')
            a = fig.add_subplot(2, 3, 5)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 6)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')
            plt.show()

        image = torch.from_numpy(image.copy()).float()
        image = image.unsqueeze(0)
        if not self.mask_dir is None:
            mask = torch.from_numpy(mask.copy()).float()
            mask = mask.unsqueeze(0)
        else:
            mask = torch.zeros((1))

        label = np.zeros(self.num_classes)
        label[self.class_label] = 1

        if self.get_id:
            label = [label, img_name.split('.')[0]]
            if not self.mask_brain_path is None:
                mask_brain = torch.from_numpy(mask_brain.copy()).float()
                mask_brain = mask_brain.unsqueeze(0)
                label = label + [mask_brain]

        return image, label, mask



class ADNIRegDataset(torch.utils.data.Dataset):
    """3D MRI dataset loader- ADNI"""

    def __init__(self, image_path='/data/ADNI_carole',
                 mask_path=None,
                 mask_brain_path=None,
                 num_classes=2,
                 class_label=0,
                 get_id=False,
                 group='train',
                 labels_path='/data/ADNI_carole/ConversionProject/ADNI_mmse_labels.pkl',
                 transform=None):
        """
        Args:
            image_path (string): path to image folder.
            label_path (string): path to labels file.
            transform (callable, optional): Optional transforms to be applied on a sample.
        """
        self.transform = transform
        self.num_classes = num_classes
        self.class_label = class_label
        self.img_dir = image_path
        self.image_paths = sorted(os.listdir(self.img_dir))
        temp_paths = sorted(os.listdir(self.img_dir))
        self.mask_dir = mask_path
        self.mask_brain_path = mask_brain_path
        self.get_id = get_id
        self.labels = pd.read_pickle(labels_path)

        print(len(self.image_paths))
        id_list = sorted(self.image_paths)
        remove_ind = []
        i = 0
        # check which images are present in labels
        for img in id_list:
            subject = img.split('.')[0]
            if not (any(self.labels['id'] == subject)):
                remove_ind.append(i)
            i = i + 1
        id_list = [i for j, i in enumerate(id_list) if j not in remove_ind]
        self.image_paths = sorted(id_list)

        print(len(self.image_paths))
        if group == 'train':
            for im in temp_paths:
                image_name = im.split('.')[0]
                if any(image_name in s for s in REMOVE_TEST_IDS):
                    self.image_paths.remove(im)
        print(len(self.image_paths))


    def __len__(self):
        return len(self.image_paths)

    def __getitem__(self, idx, plot=False):
        img_name = sorted(self.image_paths)[idx]
        print(img_name)

        image = nib.load(os.path.join(self.img_dir, img_name))
        image = np.float32(image.get_fdata())

        if not self.mask_dir is None:
            mask = nib.load(os.path.join(self.mask_dir, img_name))
            mask = np.float32(mask.get_fdata())
        if not self.mask_brain_path is None:
            mask_brain = nib.load(os.path.join(self.mask_brain_path, img_name))
            mask_brain = np.float32(mask_brain.get_fdata())

        if plot:
            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]
            fig = plt.figure()
            a = fig.add_subplot(2, 3, 1)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('Before aug')
            a = fig.add_subplot(2, 3, 2)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 3)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')

            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

        if self.transform:
            image = self.transform(image)
            if not self.mask_dir is None:
                mask = self.transform(mask)
            if not self.mask_brain_path is None:
                mask_brain = self.transform(mask_brain)

        if plot:
            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]

            a = fig.add_subplot(2, 3, 4)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('After aug')
            a = fig.add_subplot(2, 3, 5)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 6)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')
            plt.show()

        image = torch.from_numpy(image.copy()).float()
        image = image.unsqueeze(0)
        if not self.mask_dir is None:
            mask = torch.from_numpy(mask.copy()).float()
            mask = mask.unsqueeze(0)
        else:
            mask = torch.zeros((1))

        f = img_name.split('.')[0]
        labels = self.labels.loc[self.labels['id'] == f]
        if self.class_label ==0:
            labels = labels.loc[labels['group'] == 'MCI']
            label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])
        else:
            labels = labels.loc[labels['group'] == 'AD']
            label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])

        label = torch.from_numpy(label.copy()).float()

        if self.get_id:
            label = [label, img_name.split('.')[0]]
            if not self.mask_brain_path is None:
                mask_brain = torch.from_numpy(mask_brain.copy()).float()
                mask_brain = mask_brain.unsqueeze(0)
                label = label + [mask_brain]


        return image, label, mask




class ADNIRegDataset_old(torch.utils.data.Dataset):
    """3D MRI dataset loader- ADNI"""

    def __init__(self, image_path='/data/ADNI_carole',
                 mask_path=None,
                 mask_brain_path=None,
                 num_classes=2,
                 class_label=0,
                 get_id=False,
                 group='train',
                 labels_path='/data/ADNI_carole/ConversionProject/ADNI_labels.pkl',
                 transform=None):
        """
        Args:
            image_path (string): path to image folder.
            label_path (string): path to labels file.
            transform (callable, optional): Optional transforms to be applied on a sample.
        """
        self.transform = transform
        self.num_classes = num_classes
        self.class_label = class_label
        self.img_dir = image_path
        self.image_paths = sorted(os.listdir(self.img_dir))
        temp_paths = sorted(os.listdir(self.img_dir))
        self.mask_dir = mask_path
        self.mask_brain_path = mask_brain_path
        self.get_id = get_id
        self.labels = pd.read_pickle(labels_path)

        # print(len(self.image_paths))
        REMOVE_AD_MCI_NO_MMSE_SUB = REMOVE_AD_NO_MMSE_SUB + REMOVE_MCI_NO_MMSE_SUB
        for im in temp_paths:
            image_name = im.split('.')[0]
            if any(image_name in s for s in REMOVE_AD_MCI_NO_MMSE_SUB):
                self.image_paths.remove(im)

        # print(len(self.image_paths))
        if group == 'train':
            for im in temp_paths:
                image_name = im.split('.')[0]
                if any(image_name in s for s in REMOVE_TEST_IDS):
                    self.image_paths.remove(im)
        else:
            for im in temp_paths:
                image_name = im.split('.')[0]
                if any(image_name in s for s in REMOVE_CONV_SUB):
                    self.image_paths.remove(im)
        # print(len(self.image_paths))
        # print(self.image_paths)

    def __len__(self):
        return len(self.image_paths)

    def __getitem__(self, idx, plot=False):
        img_name = sorted(self.image_paths)[idx]
        print(img_name)

        image = nib.load(os.path.join(self.img_dir, img_name))
        image = np.float32(image.get_fdata())

        if not self.mask_dir is None:
            mask = nib.load(os.path.join(self.mask_dir, img_name))
            mask = np.float32(mask.get_fdata())
        if not self.mask_brain_path is None:
            mask_brain = nib.load(os.path.join(self.mask_brain_path, img_name))
            mask_brain = np.float32(mask_brain.get_fdata())

        if plot:
            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]
            fig = plt.figure()
            a = fig.add_subplot(2, 3, 1)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('Before aug')
            a = fig.add_subplot(2, 3, 2)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 3)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')

            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

        if self.transform:
            image = self.transform(image)
            if not self.mask_dir is None:
                mask = self.transform(mask)
            if not self.mask_brain_path is None:
                mask_brain = self.transform(mask_brain)

        if plot:
            print(np.min(image))
            print(np.max(image))
            print(np.mean(image))
            print(np.std(image))

            ind = 50
            sag_slice = image[:, :, ind]
            cor_slice = image[:, ind, :]
            axi_slice = image[ind, :, :]

            a = fig.add_subplot(2, 3, 4)
            imgplot = plt.imshow(np.rot90(sag_slice))
            a.axis('off')
            a.set_title('After aug')
            a = fig.add_subplot(2, 3, 5)
            imgplot = plt.imshow(np.rot90(cor_slice))
            a.axis('off')
            a = fig.add_subplot(2, 3, 6)
            imgplot = plt.imshow(np.rot90(axi_slice))
            a.axis('off')
            plt.show()

        image = torch.from_numpy(image.copy()).float()
        image = image.unsqueeze(0)
        if not self.mask_dir is None:
            mask = torch.from_numpy(mask.copy()).float()
            mask = mask.unsqueeze(0)
        else:
            mask = torch.zeros((1))

        f = img_name.split('.')[0]
        # subject = int(f)
        labels = self.labels.loc[self.labels['id'] == f]
        if self.class_label == 0:
            label = np.array([labels['MMSCORE'].to_numpy().astype(float)[0]])
        else:
            label = np.array([labels['MMSCORE'].to_numpy().astype(float)[-1]])
        label = torch.from_numpy(label.copy()).float()

        if self.get_id:
            label = [label, img_name.split('.')[0]]
            if not self.mask_brain_path is None:
                mask_brain = torch.from_numpy(mask_brain.copy()).float()
                mask_brain = mask_brain.unsqueeze(0)
                label = label + [mask_brain]


        return image, label, mask
